% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-summary.R
\name{summarize_column}
\alias{summarize_column}
\title{Summarize a Single Column}
\usage{
summarize_column(x)
}
\arguments{
\item{x}{A vector to summarize.}
}
\value{
A named character vector with summary statistics including:
type, unique count, missing count, missing share (proportion from 0 to 1),
most frequent value (for non-numeric), mean, sd, min, quartiles (q25, q50,
q75), max, and three example values.
}
\description{
Computes summary statistics for a vector. Handles numeric, character,
factor, logical, Date, and other types with appropriate statistics for each.
}
\examples{
summarize_column(c(1, 2, 3, NA, 5))
summarize_column(c("a", "b", "a", "c"))

}
\seealso{
Other data quality:
\code{\link[=audit_transform]{audit_transform()}},
\code{\link[=diagnose_nas]{diagnose_nas()}},
\code{\link[=diagnose_strings]{diagnose_strings()}},
\code{\link[=get_summary_table]{get_summary_table()}},
\code{\link[=tab]{tab()}}
}
\concept{data quality}
